(**********************************)
(* Lecture du fichier INSTALL.INS *)
(**********************************)
unit Lecture;

interface

uses Dialogs,LZExpand;

const Title_Parag : array [1 .. 10] of string =
                    ('INFORMATION', (* Info about program to install       *)
                     'DISKS',       (* Number of disk                      *)
                     'ORIGIN',      (* Origin informations                 *)
                     'DESTINATION', (* Destination informations            *)
                     'GROUP',       (* Groups to create                    *)
                     'PACKET FILES',(* Description of all packet file name *)
                     'ICONS',       (* Icons to create                     *)
                     'FILES',       (* Files to copy                       *)
                     'RUN',         (* Files to run                        *)
                     'INI');
      Number_Para = 10;             (* number of <> paragraphs              *)
      lgn_Remarks = ';';            (* ligne en remarque                   *)
      lgn_Paragrf = '[';            (* marque de paragraphe                *)
      lgn_EqualSi = '=';            (* marque de debut des parametres      *)
      lgn_Separat = ',';            (* marque de separateur des parametres *)
      lgn_Pat_Red = '%';            (* marque d'indicateur de repertoire   *)
      lgn_Pat_Trg = '1';            (* repertoire de destination           *)
      lgn_Pat_Win = 'W';            (* repertoire de Windows               *)
      lgn_Pat_Sys = 'S';            (* repertoire de Windows/System        *)
      lgn_Pat_Rot = 'R';            (* repertoire racine                   *)
      exefilename = 'SETUP.EXE';    (* current Exe file name               *)

(* suppress space on right *)
function TRIMString (Line : string) : string;
(* determine compressed file *)
function IsCompressed (sFile : string) : boolean;
(* set a new path in array and return position*)
function SetPath (sPath : string) : integer;

(* lecture du fichier d'install *)
function ReadInsFile : integer;
         {00 : File readed OK
          01 : InsFile not found
          02 : Unabled to open InsFile
          03 : Unabled to read one line
          04 : Wrong paragraf title
          05 : Wrong line because = sign not found
          06 : Wrong line because paragrf not found
          07 : Wrong line in a information paragraf
          08 : Wrong line in a disks paragraf
          09 : Wrong line in a origin paragraf
          10 : Wrong line in a destination paragraf
          11 : Wrong line in a group paragraf
          12 : Wrong line in a icons paragraf
          13 : Wrong line in a files paragraf
          14 : Wrong line in a run paragraf
          15 : Wrong number in Num parameter
          16 : Wrong number before = sign
          17 : Wrong parameters in a Group line
          18 : Wrong parameters in a Icons line
          19 : Wrong parameters in a Files line
          20 : Wrong parameters in a Run line
          21 : Wrong number in a Dsk line
          22 : Wring number in a Spc line
          23 : Wrong line in a Ini paragraf
          24 : Wrong number in a Ini parameter
          25 : Wrong number before = sign
          26 : Wrong parameter in a INI line
          27 : Wrong line in a packet file paragraf
          28 : Wrong parameter in a packet file line}

implementation

uses SysUtils,Decla,Disque;

(***************************)
(* suppress space on right *)
(***************************)
function TRIMString (Line : string) : string;
var Temp : string;
begin
 Temp := line;
 while Temp [length (Temp)] = ' ' do
  temp := copy (Temp,1,length (temp) - 1);
 TRIMString := Temp;
end;

(*****************************)
(* determine compressed file *)
(*****************************)
function IsCompressed (sFile : string) : boolean;
var bReturn      : boolean;
    pFileName     : Array [0..256] of Char;
    pInternalName : Array [0..256] of Char;
    iTmp          : integer;
begin
 bReturn := false;
 if FileExists (sFile) then
 begin
  StrPCopy (pFileName,sFile);
  iTmp := GetExpandedName (pFileName,pInternalName);
  if iTmp < 0 then
   bReturn := false
  else
  begin
   if (StrEnd (pInternalName) - pInternalName) > 0 then
    if ExtractFileName (sFile) = ExtractFileName (StrPas (pInternalName)) then
     bReturn := false
    else
     bReturn := true;
  end;
 end;
 IsCompressed := bReturn;
end;

(**********************************************)
(* set a new path in array and return position*)
(**********************************************)
function SetPath (sPath : string) : integer;
var iReturn  : integer;
    iCtr     : integer;
begin
 iReturn := 0;
 for iCtr := 1 to Max_Path do
 begin
  if VPath [iCtr].PathDriv = UpperCase (Copy (sPath,3,length (sPath))) then
  begin
   iReturn := iCtr;
   break;
  end
  else
  begin
   if vPath [iCtr].PathDriv = '' then
   begin
    iReturn := iCtr;
    VPath [iCtr].LettDriv := UpperCase (Copy (sPath,1,2));
    VPath [iCtr].PathDriv  := UpperCase (Copy (sPath,3,length (sPath)));
    break;
   end;
  end;
 end;
 SetPath := iReturn;
end;


(*******************************)
(* traitements des lignes lues *)
(*******************************)

(* analyse du nombre de disque *)
function Traitement_NombreDsk (Ligne : string) : boolean;
var Tmp : boolean;
    Err : integer;
begin
 Tmp := true;
 val (Ligne,Disk_Number,Err);
 Tmp := (Disk_Number > 0) and (Err = 0);
 Traitement_NombreDsk := Tmp;
end;

(* analyse de l'espace requis *)
function Traitement_SpaceNeed (Ligne : string) : boolean;
var Tmp : boolean;
    Err : integer;
begin
 Tmp := true;
 val (Ligne,Disk_Space,Err);
 Tmp := (Disk_Number > 0) and (Err = 0);
 Traitement_SpaceNeed := Tmp;
end;

(* analyse du disque *)
function Traitement_Disque (Ligne : string;Numero : integer) : boolean;
begin
 if (Numero = 1) and (Ligne = '') then
  VPath [1].LettDriv := GetCurrentDrive (ExeFileName)
 else
 begin
  with VPath [Numero] do
  begin
   if Copy (Ligne [1], length (Ligne [1]),1) <> ':' then
    LettDriv := Ligne [1] + ':'
   else
    LettDriv := Ligne [1];
  end;
 end;
 Traitement_Disque := true;
end;

(* analyse du chemin *)
function Traitement_Chemin (Ligne : string;Numero : integer) : boolean;
begin
 if (Numero = 1) and (Ligne = '') then
  VPath [1].PathDriv := GetCurrentPath (ExeFileName)
 else
 begin
  with VPath [Numero] do
   PathDriv := Ligne;
 end;
 Traitement_Chemin := true;
end;

(* prendre les parametres *)
function Coupe (var Ligne : string) : string;
var Tmp : string;
begin
 Tmp := '';
 Tmp := copy (Ligne,1,pos (lgn_Separat,Ligne) - 1);
 Delete (Ligne,1,pos (lgn_Separat,Ligne));
 Coupe := Tmp;
end;

(* analyse d'une ligne de groupe *)
function Traitement_Groupe (Ligne : string;Numero : integer) : boolean;
var Tmp : boolean;
    Lg1 : string;
    Lg2 : string;
begin
 Tmp := true;
 Delete (Ligne,1,pos (lgn_EqualSi,Ligne));
 Lg1 := Coupe (Ligne); (* group name *)
 Lg2 := Ligne;         (* group file *)
 if Numero > Max_Group then
  Tmp := false
 else
 begin
  if not ((Lg1 = '') or (Lg2 = '')) then
  begin
   With VGroup [Numero] do
   begin
    GroupName := Lg1;
    GroupFile := UpperCase (WinDir) + '\' + UpperCase (Lg2);
   end;
  end
  else
   Tmp := False;
 end;
 Traitement_Groupe := Tmp;
end;

(* anayse of packet file *)
function Traitement_Packet (Ligne : string;Numero : integer) : boolean;
var Tmp : boolean;
    Lg1 : string;
begin
 Tmp := true;
 Delete (Ligne,1,pos (lgn_EqualSi,Ligne));
 Lg1 := Ligne;
 if Numero > Max_Pack then
  Tmp := false
 else
 begin
  if Lg1 <> '' then
   VPack [Numero].libelle := Lg1
  else
   Tmp := False;
 end;
 Traitement_Packet := Tmp;
end;

(* analyse d'une ligne d'icone *)
function Traitement_Icone (Ligne : string;Numero : integer) : boolean;
var Tmp : boolean;
    Lg1 : string;
    Lg2 : string;
    Lg3 : string;
    lg4 : string;
    lg5 : string;
    lg6 : string;
    Num : integer;
    Grp : integer;
    Err : integer;
    iPosition : integer;
begin
 Tmp := true;
 Delete (Ligne,1,pos (lgn_EqualSi,Ligne));
 Lg1 := Coupe (Ligne); (* Number       *)
 Lg2 := Coupe (Ligne); (* File Name    *)
 Lg3 := Coupe (Ligne); (* Path of file *)
 Lg4 := Coupe (Ligne); (* Description  *)
 Lg5 := Coupe (Ligne); (* Group number *)
 Lg6 := Ligne;         (* packet file  *)
 Val (Lg1,Num,Err);
 Val (Lg5,Grp,Err);
 if (Grp < 1) or (Grp > Max_Group) then Grp := 1;
 if not ((Lg1 = '') or (Lg2 = '') or (Lg3 = '') or
         (Lg4 = '') or (Lg5 = '') or (Lg6 = '')) then
 begin
  With VIcons [Numero] do
  begin
   DiskNumb := Num;
   GroupNum := Grp;
   FileName := UpperCase (Lg2);
   val (Lg6,PackFile,Err);
   if Lg3 [1] = lgn_Pat_Red then
   begin
    case Lg3 [2] of
     lgn_Pat_Win : FilePath := '%W';
     lgn_Pat_Sys : FilePath := '%S';
     lgn_Pat_Rot : FilePath := '%R';
    else
     begin
      if Lg3 [2] in ['1' .. '9'] then
       FilePath := '%' + Lg3 [2]
      else
       FilePath := '%1';
     end;
    end;
   end
   else
   begin
    iPosition := SetPath (UpperCase (Lg3));
    FilePath := '%' + IntToStr (iPosition - 1 );
   end;
   if IsCompressed (VPath [1].LettDriv +
                    VPath [1].PathDriv + '\' + FileName) then
    Compress := 1;
   IconName := Lg4;
  end;
 end
 else
  Tmp := False;
 Traitement_Icone := Tmp;
end;

(* analyse d'une ligne de fichier *)
function Traitement_Fichier (Ligne : string;Numero : integer) : boolean;
var Tmp : boolean;
    Lg1 : string;
    Lg2 : string;
    Lg3 : string;
    Lg4 : string;
    Num : integer;
    Err : integer;
    iPosition : integer;
begin
 Tmp := true;
 Delete (Ligne,1,pos (lgn_EqualSi,Ligne));
 Lg1 := Coupe (Ligne); (* Number       *)
 Lg2 := Coupe (Ligne); (* File name    *)
 Lg3 := Coupe (Ligne); (* Path of file *)
 Lg4 := Ligne;         (* Packet file  *)
 Val (Lg1,Num,Err);
 if Numero > Max_Files then
  Tmp := false
 else
 begin
  if not ((Lg1 = '') or (Lg2 = '') or (lg3 = '') or (Lg4 = '')) then
  begin
   With VFiles [Numero] do
   begin
    DiskNumb := Num;
    FileName := UpperCase (Lg2);
    val (Lg4,PackFile,Err);
    if Lg3 [1] = lgn_Pat_Red then
    begin
     case Lg3 [2] of
      lgn_Pat_Win : FilePath := '%W';
      lgn_Pat_Sys : FilePath := '%S';
      lgn_Pat_Rot : FilePath := '%R';
     else
      if Lg3 [2] in ['1' .. '9'] then
       FilePath := '%' + Lg3 [2]
      else
       FilePath := '%1';
     end;
    end
    else
    begin
     iPosition := SetPath (UpperCase (Lg3));
     FilePath := '%' + IntToStr (iPosition - 1);
    end;
    if IsCompressed (VPath [1].LettDriv +
                     VPath [1].PathDriv + '\' + FileName ) then
     Compress := 1;
   end;
  end
  else
   Tmp := False;
 end;
 Traitement_Fichier := Tmp;
end;

(* analyse d'une ligne de lancement *)
function Traitement_Run (Ligne : string;Numero : integer) : boolean;
var Tmp : boolean;
    Lg1 : string;
    Lg2 : string;
    Lg3 : string;
    Lg4 : string;
    Lg5 : string;
    Err : integer;
    iPosition : integer;
begin
 Tmp := true;
 Delete (Ligne,1,pos (lgn_EqualSi,Ligne));
 Lg1 := Coupe (Ligne); (* applicatio name     *)
 Lg2 := Coupe (Ligne); (* path of application *)
 Lg3 := Coupe (Ligne); (* documetn name       *)
 Lg4 := Coupe (Ligne); (* path of document    *)
 Lg5 := Ligne;         (* packet file         *)
 if Numero > Max_Run then
  Tmp := false
 else
 begin
  if not ((Lg1 = '') or (lg2 = '') or (lg3 = '') or
          (lg4 = '') or (lg5 = '')) then
  begin
   With VRun [Numero] do
   begin
    val (Lg5,PackFile,Err);
    FileName := UpperCase (Lg1);
    if Lg2 [1] = lgn_Pat_Red then
    begin
     case Lg2 [2] of
      lgn_Pat_Win : FilePath := '%W';
      lgn_Pat_Sys : FilePath := '%S';
      lgn_Pat_Rot : FilePath := '%R';
     else
      if Lg2 [2] in ['1' .. '9'] then
       FilePath := '%' + Lg2 [2]
      else
       FilePath := '%1';
     end;
    end
    else
    begin
     iPosition := SetPath (UpperCase (Lg2));
     FilePath := '%' + IntToStr (iPosition - 1 );
    end;
    DocsName := UpperCase (Lg3);
    if Lg4 [1] = lgn_Pat_Red then
    begin
     case Lg4 [2] of
      lgn_Pat_Win : DocsPath := UpperCase (WinDir);
      lgn_Pat_Sys : DocsPath := UpperCase (SysDir);
      lgn_Pat_Rot : FilePath := '\';
     else
      begin
       if Lg4 [2] in ['1' .. '9'] then
        DocsPath := '%' + Lg4 [2]
       else
        DocsPath := '%1';
      end;
     end;
    end
    else
    begin
     iPosition := SetPath (UpperCase (Lg4));
     DocsPath := '%' + IntToStr (iPosition - 1 );
    end;
   end;
  end
  else
   Tmp := false;
 end;
 Traitement_Run := Tmp;
end;

(* traitement d'une ligne ini file *)
function Traitement_Ini (Ligne : string;Numero : integer) : boolean;
var Tmp : boolean;
    Lg1 : string;
    Lg2 : string;
    Lg3 : string;
    Lg4 : string;
    Lg5 : string;
    iPosition : integer;
begin
 Tmp := true;
 Delete (Ligne,1,pos (lgn_EqualSi,Ligne));
 Lg1 := Coupe (Ligne); (* File name    *)
 Lg2 := Coupe (Ligne); (* Path of file *)
 Lg3 := Coupe (Ligne); (* Section name *)
 Lg4 := Coupe (Ligne); (* Field name   *)
 Lg5 := Ligne;         (* Value        *)
 if Numero > Max_IniF then
  Tmp := false
 else
 begin
  With VIniF [Numero] do
  begin
   FileName := UpperCase (Lg1);
   if Lg2 [1] = lgn_Pat_Red then
   begin
    case Lg2 [2] of
     lgn_Pat_Win : FilePath := '%W';
     lgn_Pat_Sys : FilePath := '%S';
     lgn_Pat_Rot : FilePath := '%R';
    else
     if Lg2 [2] in ['1' .. '9'] then
      FilePath := '%' + Lg2 [2]
     else
      FilePath := '%1';
    end;
   end
   else
   begin
    iPosition := SetPath( UpperCase (Lg2) );
    FilePath := '%' + IntToStr( iPosition - 1 );
   end;
   IniSecti := Lg3;
   IniField := Lg4;
   IniValue := Lg5;
  end;
 end;
 Traitement_Ini := Tmp;
end;

(* initialisation des variables *)
procedure Initialise;
var Bcl : integer;
begin
 for Bcl := 1 to Max_Group do
 begin
  With VGroup [Bcl] do
  begin
   GroupName := '';
   GroupFile := '';
  end;
 end;
 for Bcl := 1 to Max_Icons do
 begin
  With VIcons [Bcl] do
  begin
   DiskNumb := 0;
   FileName := '';
   FilePath := '';
   IconName := '';
   Compress := 0;
   GroupNum := 1;
   PackFile := 0;
  end;
 end;
 for Bcl := 1 to Max_Files do
 begin
  With VFiles [Bcl] do
  begin
   DiskNumb := 0;
   FileName := '';
   FilePath := '';
   Compress := 0;
   PackFile := 0;
  end;
 end;
 for Bcl := 1 to Max_Run do
 begin
  With VRun [Bcl] do
  begin
   FileName := '';
   FilePath := '';
   DocsName := '';
   DocsPath := '';
   PackFile := 0;
  end;
 end;
 for Bcl := 1 to Max_Path do
 begin
  With VPath [Bcl] do
  begin
   LettDriv := '';
   PathDriv := '';
  end;
 end;
 for Bcl := 1 to Max_IniF do
 begin
  With VIniF [Bcl] do
  begin
   FileName := '';
   FilePath := '';
   IniSecti := '';
   IniField := '';
   IniValue := '';
  end;
 end;
 for Bcl := 1 to Max_Pack do
 begin
  With VPack [Bcl] do
  begin
   Libelle := '';
   Selecte := true;
  end;
 end;
 Number_Direc := 0;
 Number_Group := 0;
 Number_Icons := 0;
 Number_Files := 0;
 Number_Run := 0;
 Number_Ini := 0;
 Number_Pack := 0;
end;

(**********************)
(* lecture du fichier *)
(**********************)
function ReadInsFile : integer;
var Tmp : integer;
    Fch : System.Text;
    Lgn : string;
    LTm : string;
    Bcl : integer;
    Nbr : integer;
    Lop : integer;
    Err : integer;
    Nb1 : integer;
    Par : integer;      (* 1=Inf  2=Dsk  3=Org  4=Trg *)
                        (* 5=Grp  6=Ico  7=Fil  8=Run *)
begin
 Tmp := 0;
 Par := 0;
 Initialise;
 if ExistFile (GetCurrentDrive (ExeFileName) +
               GetCurrentPath (ExeFileName) + Fch_Ins + 'INS') = 0 then
  Tmp := 1
 else
 begin
  assign (Fch,
          GetCurrentDrive (ExeFileName) +
          GetCurrentPath (ExeFileName) + Fch_Ins + 'INS');
  {$I-}; reset (Fch); {$I+};
  if ioresult <> 0 then
   Tmp := 2
  else
  begin
   while not (eof (Fch)) do
   begin
    {$I-}; readln (fch,Lgn); {$I+};
    if ioresult <> 0 then
     Tmp := 3
    else
    begin
     if (lgn <> '') and (Lgn [1] <> lgn_Remarks) then
     begin
      if Lgn [1] = lgn_Paragrf then
      (* calculer le nouveau paragraphe *)
      begin
       Par := 0;
       Ltm := UpperCase (copy (Lgn,2,length (Lgn) - 2));
       for Bcl := 1 to Number_Para do
        if Ltm = Title_Parag [Bcl] then Par := Bcl;
       if Par = 0 then
        Tmp := 4;
      end
      else
      (* ligne a traiter *)
      begin
       if pos (lgn_EqualSi,lgn) = 0 then
        Tmp := 5
       else
       begin
        Ltm := UpperCase (copy (Lgn,1,pos (Lgn_EqualSi,Lgn) - 1));
        Delete (lgn,1,pos (lgn_EqualSi,Lgn));
        case Par of
         1 : (* information *)
             begin
              if Ltm = 'TITLE' then Title := Lgn
              else
               if Ltm = 'SUBTITLE' then SubTitle := Lgn
              else
               if Ltm = 'VERSION' then Version  := Lgn
              else
               if Ltm = 'AUTHOR' then Author := Lgn
              else
               if Ltm = 'COPYRIGHT' then Copyright := Lgn
              else
               Tmp := 7;
             end;
         2 : (* disks       *)
             begin
              if Ltm = 'DSK' then
              begin
               if Traitement_NombreDsk (Lgn) = false then
                Tmp := 21;
              end
              else
              if Ltm = 'SPC' then
              begin
               if Traitement_SpaceNeed (Lgn) = false then
                Tmp := 22;
              end
              else
               Tmp := 8;
             end;
         3 : (* origin      *)
             begin
              if Ltm = 'ODSK' then Traitement_Disque (Lgn,1)
              else
               if Ltm = 'OPAT' then Traitement_Chemin (Lgn,1)
              else
               Tmp := 9;
             end;
         4 : (* destination *)
             begin
              if Ltm = 'NUM' then
              begin
               val (Lgn,Nbr,Err);
               if Err <> 0 then
                Tmp := 15
               else
               begin
                Number_Direc := Nbr;
                if Number_Direc <> 0 then
                begin
                 for Lop := 1 to Nbr do
                 begin
                  {$I-}; readln (Fch,Lgn); {$I+};
                  if ioresult <> 0 then
                   Tmp := 3
                  else
                  begin
                   Ltm := UpperCase (copy(Lgn,1,pos(Lgn_EqualSi,Lgn) - 1));
                   Delete (lgn,1,pos (lgn_EqualSi,Lgn));
                   if Ltm = 'DDSK' then Traitement_Disque (Lgn,Lop + 1)
                                   else Tmp := 10;
                  end;
                  {$I-}; readln (Fch,Lgn); {$I+};
                  if ioresult <> 0 then
                   Tmp := 3
                  else
                  begin
                   Ltm := UpperCase (copy(Lgn,1,pos(Lgn_EqualSi,Lgn) - 1));
                   Delete (lgn,1,pos (lgn_EqualSi,Lgn));
                   if Ltm = 'DPAT' then Traitement_Chemin (Lgn,Lop + 1)
                                   else Tmp := 10;
                  end;
                 end;
                end;
               end;
              end;
             end;
         5 : (* group       *)
             begin
              if Ltm = 'NUM' then
              begin
               val (Lgn,Nbr,Err);
               if Err <> 0 then
                Tmp := 15
               else
               begin
                Number_group := Nbr;
                if Number_Group <> 0 then
                begin
                 for Lop := 1 to Nbr do
                 begin
                  {$I-}; readln (Fch,Lgn); {$I+};
                  if ioresult <> 0 then
                   Tmp := 3
                  else
                  begin
                   val (copy (Lgn,1,pos (lgn_EqualSi,Lgn) - 1),Nb1,Err);
                   if Nb1 <> Lop then
                    Tmp := 16
                   else
                    if Traitement_Groupe (Lgn,Nb1) = false then
                     Tmp := 17;
                  end;
                 end;
                end;
               end;
              end
              else
               Tmp := 11;
             end;
         6 : (* packet file group *)
             begin
              if Ltm = 'NUM' then
              begin
               val (Lgn,Nbr,Err);
               if Err <> 0 then
                Tmp := 15
               else
               begin
                Number_Pack := Nbr;
                if Number_Pack <> 0 then
                begin
                 for Lop := 1 to Nbr do
                 begin
                  {$I-}; readln (Fch,Lgn); {$I+};
                  if ioresult <> 0 then
                   Tmp := 3
                  else
                  begin
                   val (copy (Lgn,1,pos (lgn_EqualSi,Lgn) - 1),Nb1,Err);
                   if Nb1 <> Lop then
                    Tmp := 16
                   else
                    if Traitement_Packet (Lgn,Nb1) = false then
                     Tmp := 28;
                  end;
                 end;
                end;
               end;
              end
              else
               Tmp := 27;
             end;
         7 : (* icons       *)
             begin
              if Ltm = 'NUM' then
              begin
               val (Lgn,Nbr,Err);
               if Err <> 0 then
                Tmp := 15
               else
               begin
                Number_Icons := Nbr;
                if Number_Icons <> 0 then
                begin
                 for Lop := 1 to Nbr do
                 begin
                  {$I-}; readln (Fch,Lgn); {$I+};
                  if ioresult <> 0 then
                   Tmp := 3
                  else
                  begin
                   val (copy (Lgn,1,pos (lgn_EqualSi,Lgn) - 1),Nb1,Err);
                   if Nb1 <> Lop then
                    Tmp := 16
                   else
                    if Traitement_Icone (Lgn,Nb1) = false then
                     Tmp := 18;
                  end;
                 end;
                end;
               end;
              end
              else
               Tmp := 12;
             end;
         8 : (* files       *)
             begin
              if Ltm = 'NUM' then
              begin
               val (Lgn,Nbr,Err);
               if Err <> 0 then
                Tmp := 15
               else
               begin
                Number_Files := Nbr;
                if Number_Files <> 0 then
                begin
                 for Lop := 1 to Nbr do
                 begin
                  {$I-}; readln (Fch,Lgn); {$I+};
                  if ioresult <> 0 then
                   Tmp := 3
                  else
                  begin
                   val (copy (Lgn,1,pos (lgn_EqualSi,Lgn) - 1),Nb1,Err);
                   if Nb1 <> Lop then
                    Tmp := 16
                   else
                    if Traitement_Fichier (Lgn,Nb1) = false then
                     Tmp := 19;
                  end;
                 end;
                end;
               end;
              end
              else
               Tmp := 13;
             end;
         9 : (* run         *)
             begin
              if Ltm = 'NUM' then
              begin
               val (Lgn,Nbr,Err);
               if Err <> 0 then
                Tmp := 15
               else
               begin
                Number_Run := Nbr;
                if Number_Run <> 0 then
                begin
                 for Lop := 1 to Nbr do
                 begin
                  {$I-}; readln (Fch,Lgn); {$I+};
                  if ioresult <> 0 then
                   Tmp := 3
                  else
                  begin
                   val (copy (Lgn,1,pos (lgn_EqualSi,Lgn) - 1),Nb1,Err);
                   if Nb1 <> Lop then
                    Tmp := 16
                   else
                    if Traitement_Run (Lgn,Nb1) = false then
                     Tmp := 20;
                  end;
                 end;
                end;
               end;
              end
              else
               Tmp := 14;
             end;
         10 : (* ini file *)
             begin
              if Ltm = 'NUM' then
              begin
               val (Lgn,Nbr,Err);
               if Err <> 0 then
                Tmp := 24
               else
               begin
                Number_Ini := Nbr;
                if Number_Ini <> 0 then
                begin
                 for Lop := 1 to Nbr do
                 begin
                  {$I-}; readln (Fch,Lgn); {$I+};
                  if ioresult <> 0 then
                   Tmp := 3
                  else
                  begin
                   val (copy (Lgn,1,pos (lgn_EqualSi,Lgn) - 1),Nb1,Err);
                   if Nb1 <> Lop then
                    Tmp := 25
                   else
                    if Traitement_Ini (Lgn,Nb1) = false then
                     Tmp := 26;
                  end;
                 end;
                end;
               end;
              end
              else
               Tmp := 23;
             end;
        else
         Tmp := 6;
        end;
       end;
      end;
     end;
    end;
   end;
   Close (Fch);
  end;
 end;
 ReadInsFile := Tmp;
end;

end.
